@@ -1,5 +1,5 @@
 -- Internal.hs: private utility functions
--- Copyright © 2012-2013  Clint Adams
+-- Copyright © 2012-2014  Clint Adams
 -- This software is released under the terms of the Expat license.
 -- (See the LICENSE file).
 
@@ -12,6 +12,10 @@ module Codec.Encryption.OpenPGP.Internal (
  , issuer
  , emptyPSC
  , pubkeyToMPIs
+ , multiplicativeInverse
+ , sigType
+ , sigPKA
+ , sigHA
 ) where
 
 import Crypto.PubKey.HashDescr (HashDescr(..), hashDescrMD5, hashDescrSHA1, hashDescrSHA224, hashDescrSHA256, hashDescrSHA384, hashDescrSHA512, hashDescrRIPEMD160)
@@ -77,3 +81,23 @@ pubkeyToMPIs (DSAPubKey k) = [
   where pkParams f = MPI . f . DSA.public_params $ k
 
 pubkeyToMPIs (ElGamalPubKey k) = fmap MPI k
+
+multiplicativeInverse :: Integral a => a -> a -> a
+multiplicativeInverse _ 1 = 1
+multiplicativeInverse q p = (n * q + 1) `div` p
+    where n = p - multiplicativeInverse p (q `mod` p)
+
+sigType :: SignaturePayload -> Maybe SigType
+sigType (SigV3 st _ _ _ _ _ _) = Just st
+sigType (SigV4 st _ _ _ _ _ _) = Just st
+sigType _ = Nothing -- this includes v2 sigs, which don't seem to be specified in the RFCs but exist in the wild
+
+sigPKA :: SignaturePayload -> Maybe PubKeyAlgorithm
+sigPKA (SigV3 _ _ _ pka _ _ _) = Just pka
+sigPKA (SigV4 _ pka _ _ _ _ _) = Just pka
+sigPKA _ = Nothing -- this includes v2 sigs, which don't seem to be specified in the RFCs but exist in the wild
+
+sigHA :: SignaturePayload -> Maybe HashAlgorithm
+sigHA (SigV3 _ _ _ _ ha _ _) = Just ha
+sigHA (SigV4 _ _ ha _ _ _ _) = Just ha
+sigHA _ = Nothing -- this includes v2 sigs, which don't seem to be specified in the RFCs but exist in the wild
