@@ -20,7 +20,7 @@ import Codec.Encryption.OpenPGP.Types
 
 keySize (RSAPubKey x) = RSA.public_size x * 8
 keySize (DSAPubKey x) = bitcount . DSA.params_p . DSA.public_params $ x
-keySize (ElGamalPubKey x) = bitcount $ x !! 0
+keySize (ElGamalPubKey x) = bitcount $ head x
 
 bitcount = (*8) . length . unfoldr (\x -> if x == 0 then Nothing else Just (True, x `shiftR` 8))
 
