@@ -1,31 +1,34 @@
 -- Serialize.hs: OpenPGP (RFC4880) serialization (using cereal)
--- Copyright © 2012-2013  Clint Adams
+-- Copyright © 2012-2014  Clint Adams
 -- This software is released under the terms of the Expat license.
 -- (See the LICENSE file).
 
 module Codec.Encryption.OpenPGP.Serialize (
    putSKAddendum
+ , getSecretKey
 ) where
 
 import Control.Applicative ((<$>),(<*>))
 import Control.Lens ((^.))
-import Control.Monad (guard, mplus, replicateM)
+import Control.Monad (guard, liftM, mplus, replicateM, replicateM_)
 import qualified Crypto.PubKey.RSA as R
 import qualified Crypto.PubKey.DSA as D
 import Data.Bits ((.&.), (.|.), shiftL, shiftR, testBit)
 import Data.ByteString (ByteString)
 import qualified Data.ByteString as B
-import qualified Data.ByteString.Char8 as BC8
 import Data.List (mapAccumL)
 import Data.Serialize (Serialize, get, put)
 import Data.Serialize.Get (Get, getWord8, getWord16be, getWord32be, getBytes, getByteString, getWord16le, runGet, remaining)
 import Data.Serialize.Put (Put, putWord8, putWord16be, putWord32be, putByteString, putWord16le, runPut)
 import Data.Set (Set)
 import qualified Data.Set as Set
+import qualified Data.Text as T
+import Data.Text.Encoding (decodeUtf8With, encodeUtf8)
+import Data.Text.Encoding.Error (lenientDecode)
 import Data.Word (Word8, Word32)
 import Data.Maybe (fromMaybe)
 
-import Codec.Encryption.OpenPGP.Internal (countBits, beBSToInteger, integerToBEBS, pubkeyToMPIs)
+import Codec.Encryption.OpenPGP.Internal (countBits, beBSToInteger, integerToBEBS, pubkeyToMPIs, multiplicativeInverse)
 import Codec.Encryption.OpenPGP.Types
 
 instance Serialize SigSubPacket where
@@ -36,19 +39,19 @@ instance Serialize SigSubPacket where
 --     put = putNotationFlagSet
 
 instance Serialize CompressionAlgorithm where
-    get = getWord8 >>= return . toFVal
+    get = liftM toFVal getWord8
     put = putWord8 . fromFVal
 
 instance Serialize PubKeyAlgorithm where
-    get = getWord8 >>= return . toFVal
+    get = liftM toFVal getWord8
     put = putWord8 . fromFVal
 
 instance Serialize HashAlgorithm where
-    get = getWord8 >>= return . toFVal
+    get = liftM toFVal getWord8
     put = putWord8 . fromFVal
 
 instance Serialize SymmetricAlgorithm where
-    get = getWord8 >>= return . toFVal
+    get = liftM toFVal getWord8
     put = putWord8 . fromFVal
 
 instance Serialize MPI where
@@ -56,7 +59,7 @@ instance Serialize MPI where
     put = putMPI
 
 instance Serialize SigType where
-    get = getWord8 >>= return . toFVal
+    get = liftM toFVal getWord8
     put = putWord8 . fromFVal
 
 instance Serialize UserAttrSubPacket where
@@ -217,7 +220,7 @@ getSigSubPacket = do
                        return $ SigSubPacket crit (PreferredKeyServer pks)
             | pt == 25 = do
                        primacy <- get
-                       return $ SigSubPacket crit (PreferredKeyServer primacy)
+                       return $ SigSubPacket crit (PrimaryUserId primacy)
             | pt == 26 = do
                        url <- getByteString (l - 1)
                        return $ SigSubPacket crit (PolicyURL url)
@@ -226,7 +229,7 @@ getSigSubPacket = do
                        return $ SigSubPacket crit (KeyFlags (bsToFFSet kfs))
             | pt == 28 = do
                        uid <- getByteString (l - 1)
-                       return $ SigSubPacket crit (SignersUserId (BC8.unpack uid))
+                       return $ SigSubPacket crit (SignersUserId (T.unpack . decodeUtf8With lenientDecode $ uid))
             | pt == 29 = do
                        rcode <- getWord8
                        rreason <- getByteString (l - 2)
@@ -333,7 +336,7 @@ putSigSubPacket (SigSubPacket crit (KeyFlags kfs)) = do
     putSigSubPacketType crit 27
     putByteString kbs
 putSigSubPacket (SigSubPacket crit (SignersUserId userid)) = do
-    let bs = BC8.pack userid
+    let bs = encodeUtf8 . T.pack $ userid
     putSubPacketLength . fromIntegral $ (1 + B.length bs)
     putSigSubPacketType crit 28
     putByteString bs
@@ -391,14 +394,14 @@ putSubPacketLength l
 getSigSubPacketType :: Get (Bool, Word8)
 getSigSubPacketType = do
                          x <- getWord8
-                         if x .&. 0x80 == 0x80 then return (True, x .&. 0x7f) else return (False, x)
+                         return (if x .&. 128 == 128 then (True, x .&. 127) else (False, x))
 
 putSigSubPacketType :: Bool -> Word8 -> Put
 putSigSubPacketType False sst = putWord8 sst
 putSigSubPacketType True sst = putWord8 (sst .|. 0x80)
 
 bsToFFSet :: FutureFlag a => ByteString -> Set a
-bsToFFSet bs = Set.fromAscList .  concat . snd $ mapAccumL (\acc y -> (acc+8, concatMap (\x -> if y .&. (shiftR 128 x) == (shiftR 128 x) then [toFFlag (acc + x)] else []) [0..7])) 0 (B.unpack bs)
+bsToFFSet bs = Set.fromAscList .  concat . snd $ mapAccumL (\acc y -> (acc+8, concatMap (\x -> if y .&. shiftR 128 x == shiftR 128 x then [toFFlag (acc + x)] else []) [0..7])) 0 (B.unpack bs)
 
 ffSetToFixedLengthBS :: (Integral a, FutureFlag b) => a -> Set b -> ByteString
 ffSetToFixedLengthBS len ffs = B.take (fromIntegral len) (B.append (ffSetToBS ffs) (B.pack (replicate 5 0)))
@@ -407,7 +410,9 @@ ffSetToBS :: FutureFlag a => Set a -> ByteString
 ffSetToBS = B.pack . ffSetToBS'
     where
         ffSetToBS' :: FutureFlag a => Set a -> [Word8]
-        ffSetToBS' ks = map (foldl (.|.) 0 . map (shiftR 128 . flip mod 8 . fromFFlag) . Set.toAscList) (map (\x -> Set.filter (\y -> fromFFlag y `div` 8 == x) ks) [0..(fromFFlag $ Set.findMax ks) `div` 8])
+        ffSetToBS' ks
+          | Set.null ks = [] -- FIXME: should this be [0]?
+          | otherwise = map ((foldl (.|.) 0 .  map (shiftR 128 . flip mod 8 . fromFFlag) . Set.toAscList) . (\ x -> Set.filter (\ y -> fromFFlag y `div` 8 == x) ks)) [0 .. fromFFlag (Set.findMax ks) `div` 8]
 
 fromS2K :: S2K -> ByteString
 fromS2K (Simple hashalgo) = B.pack [0, fromIntegral . fromFVal $ hashalgo]
@@ -415,7 +420,7 @@ fromS2K (Salted hashalgo salt)
     | B.length salt == 8 = B.pack [1, fromIntegral . fromFVal $ hashalgo] `B.append` salt
     | otherwise = error "Confusing salt size"
 fromS2K (IteratedSalted hashalgo salt count)
-    | B.length salt == 8 = B.pack [3, fromIntegral . fromFVal $ hashalgo] `B.append` salt `B.snoc` (encodeIterationCount count)
+    | B.length salt == 8 = B.pack [3, fromIntegral . fromFVal $ hashalgo] `B.append` salt `B.snoc` encodeIterationCount count
     | otherwise = error "Confusing salt size"
 fromS2K (OtherS2K _ bs) = bs
 
@@ -502,7 +507,7 @@ getPkt :: Get Pkt
 getPkt = do
     (t, pl) <- getPacketTypeAndPayload
     case runGet (getPkt' t (B.length pl)) pl of
-        Left e -> fail e
+        Left e -> return $! BrokenPacketPkt e t pl
         Right p -> return p
     where
         getPkt' :: Word8 -> Int -> Get Pkt
@@ -514,13 +519,13 @@ getPkt = do
                           remainder <- remaining
                           mpib <- getBytes remainder
                           case runGet (many getMPI) mpib of
-                              Left e -> error e
+                              Left e -> fail ("PKESK MPIs " ++ e)
                               Right sk -> return $ PKESKPkt pv (EightOctetKeyId eokeyid) (toFVal pkalgo) sk
             | t == 2 = do
                           remainder <- remaining
                           bs <- getBytes remainder
                           case runGet get bs of
-                              Left e -> error e
+                              Left e -> fail ("signature packet " ++ e)
                               Right sp -> return $ SignaturePkt sp
             | t == 3 = do
                           pv <- getWord8
@@ -543,7 +548,7 @@ getPkt = do
                                                        ska <- getSKAddendum pkp
                                                        return $ SecretKeyPkt pkp ska
                           case ps of
-                              Left err -> error err
+                              Left err -> fail ("secret key " ++ err)
                               Right key -> return key
             | t == 6 = do
                           pkp <- getPKPayload
@@ -554,7 +559,7 @@ getPkt = do
                                                        ska <- getSKAddendum pkp
                                                        return $ SecretSubkeyPkt pkp ska
                           case ps of
-                              Left err -> error err
+                              Left err -> fail ("secret subkey " ++ err)
                               Right key -> return key
             | t == 8 = do
                           ca <- getWord8
@@ -571,21 +576,21 @@ getPkt = do
                           flen <- getWord8
                           fn <- getByteString (fromIntegral flen)
                           ts <- getWord32be
-                          ldata <- getByteString (len - (6 + (fromIntegral flen)))
+                          ldata <- getByteString (len - (6 + fromIntegral flen))
                           return $ LiteralDataPkt (toFVal dt) fn ts ldata
             | t == 12 = do
                           tdata <- getByteString len
                           return $ TrustPkt tdata
             | t == 13 = do
                           udata <- getBytes len
-                          return $ UserIdPkt (BC8.unpack udata)
+                          return . UserIdPkt . T.unpack . decodeUtf8With lenientDecode $ udata
             | t == 14 = do
                           pkp <- getPKPayload
                           return $ PublicSubkeyPkt pkp
             | t == 17 = do
                         bs <- getBytes len
                         case runGet (many getUserAttrSubPacket) bs of
-                            Left err -> error err
+                            Left err -> fail ("user attribute " ++ err)
                             Right uas -> return $ UserAttributePkt uas
             | t == 18 = do
                           pv <- getWord8 -- should be 1
@@ -612,7 +617,7 @@ getUserAttrSubPacket = do
                               iformat <- getWord8
                               nuls <- getBytes 12 -- should be NULs
                               bs <- getByteString (l - 17)
-                              if hver /= 1 || nuls /= (B.pack (replicate 12 0)) then fail "Corrupt UAt subpacket" else return $ ImageAttribute (ImageHV1 (toFVal iformat)) bs
+                              if hver /= 1 || nuls /= B.pack (replicate 12 0) then fail "Corrupt UAt subpacket" else return $ ImageAttribute (ImageHV1 (toFVal iformat)) bs
                 | otherwise = do
                                  bs <- getByteString (l - 1)
                                  return $ OtherUASub t bs
@@ -628,7 +633,7 @@ putUserAttrSubPacket ua = do
             putWord16le 16
             putWord8 1
             putWord8 (fromFVal iformat)
-            mapM_ putWord8 $ replicate 12 0
+            replicateM_ 12 $ putWord8 0
             putByteString idata
         putUserAttrSubPacket' (OtherUASub t bs) = do
             putWord8 t
@@ -638,7 +643,7 @@ putPkt :: Pkt -> Put
 putPkt (PKESKPkt pv eokeyid pkalgo mpis) = do
     putWord8 (0xc0 .|. 1)
     let bsk = runPut $ mapM_ put mpis
-    putPacketLength . fromIntegral $ 10 + (B.length bsk)
+    putPacketLength . fromIntegral $ 10 + B.length bsk
     putWord8 pv -- must be 3
     putByteString (unEOKI eokeyid) -- must be 8 octets
     putWord8 $ fromIntegral . fromFVal $ pkalgo
@@ -652,7 +657,7 @@ putPkt (SKESKPkt pv symalgo s2k mesk) = do
     putWord8 (0xc0 .|. 3)
     let bs2k = fromS2K s2k
     let bsk = fromMaybe B.empty mesk
-    putPacketLength . fromIntegral $ 2 + (B.length bs2k) + (B.length bsk)
+    putPacketLength . fromIntegral $ 2 + B.length bs2k + B.length bsk
     putWord8 pv -- should be 4
     putWord8 $ fromIntegral . fromFVal $ symalgo
     putByteString bs2k
@@ -714,7 +719,7 @@ putPkt (TrustPkt b) = do
     putByteString b
 putPkt (UserIdPkt u) = do
     putWord8 (0xc0 .|. 13)
-    let bs = BC8.pack u
+    let bs = encodeUtf8 . T.pack $ u
     putPacketLength . fromIntegral $ B.length bs
     putByteString bs
 putPkt (PublicSubkeyPkt pkp) = do
@@ -729,7 +734,7 @@ putPkt (UserAttributePkt us) = do
     putByteString bs
 putPkt (SymEncIntegrityProtectedDataPkt pv b) = do
     putWord8 (0xc0 .|. 18)
-    putPacketLength . fromIntegral $ (B.length b) + 1
+    putPacketLength . fromIntegral $ B.length b + 1
     putWord8 pv -- should be 1
     putByteString b
 putPkt (ModificationDetectionCodePkt hash) = do
@@ -740,10 +745,11 @@ putPkt (OtherPacketPkt t payload) = do
     putWord8 (0xc0 .|. t) -- FIXME: restrict t
     putPacketLength . fromIntegral . B.length $ payload
     putByteString payload
+putPkt (BrokenPacketPkt _ t payload) = putPkt (OtherPacketPkt t payload)
 
 getMPI :: Get MPI
 getMPI = do mpilen <- getWord16be
-            bs <- getByteString ((fromIntegral (mpilen - 1) `div` 8) + 1)
+            bs <- getByteString (fromIntegral (mpilen + 7) `div` 8)
             return $ MPI (beBSToInteger bs)
 
 getPubkey :: PubKeyAlgorithm -> Get PKey
@@ -762,9 +768,10 @@ getPubkey ForbiddenElgamal = do MPI p <- get
                                 MPI g <- get
                                 MPI y <- get
                                 return $ ElGamalPubKey [p,g,y]
-getPubkey t = fail ("Unsupported pubkey type " ++ show t)
+getPubkey _ = UnknownPKey <$> (getByteString =<< remaining)
 
 putPubkey :: PKey -> Put
+putPubkey (UnknownPKey bs) = put bs
 putPubkey p = mapM_ put (pubkeyToMPIs p)
 
 getSecretKey :: PKPayload -> Get SKey
@@ -786,6 +793,11 @@ getSecretKey pkp
         do MPI x <- get
            return $ ElGamalPrivateKey [x]
 
+putSKey :: SKey -> Put
+putSKey (RSAPrivateKey (R.PrivateKey _ d p q _ _ _)) = put (MPI d) >> put (MPI p) >> put (MPI q) >> put (MPI u)
+    where
+        u = multiplicativeInverse q p
+
 putMPI :: MPI -> Put
 putMPI (MPI i) = do let bs = integerToBEBS i
                     putWord16be . countBits $ bs
@@ -849,6 +861,10 @@ putSKAddendum (SUSSHA1 symenc s2k iv encryptedblock) = do
     put s2k
     putByteString iv
     putByteString encryptedblock
+putSKAddendum (SUUnencrypted sk checksum) = do
+    putWord8 0
+    putSKey sk
+    putWord16be checksum
 putSKAddendum _ = fail "Type not supported"
 
 symEncBlockSize :: SymmetricAlgorithm -> Int
@@ -1141,7 +1157,7 @@ getSignaturePayload = do
             remainder <- remaining
             mpib <- getBytes remainder
             case runGet (many getMPI) mpib of
-                Left e -> error e
+                Left e -> fail ("v3 sig MPIs " ++ e)
                 Right mpis -> return $ SigV3 (toFVal st) ctime (EightOctetKeyId eok) (toFVal pka) (toFVal ha) left16 mpis
         4 -> do
             st <- getWord8
@@ -1150,18 +1166,18 @@ getSignaturePayload = do
             hlen <- getWord16be
             hb <- getBytes (fromIntegral hlen)
             let hashed = case runGet (many getSigSubPacket) hb of
-                            Left err -> error err
+                            Left err -> fail ("v4 sig hasheds " ++ err)
                             Right h -> h
             ulen <- getWord16be
             ub <- getBytes (fromIntegral ulen)
             let unhashed = case runGet (many getSigSubPacket) ub of
-                            Left err -> error err
+                            Left err -> fail ("v4 sig unhasheds " ++ err)
                             Right u -> u
             left16 <- getWord16be
             remainder <- remaining
             mpib <- getBytes remainder
             case runGet (many getMPI) mpib of
-                    Left e -> error e
+                    Left e -> fail ("v4 sig MPIs " ++ e)
                     Right mpis -> return $ SigV4 (toFVal st) (toFVal pka) (toFVal ha) hashed unhashed left16 mpis
         _ -> do
             remainder <- remaining
@@ -1171,6 +1187,7 @@ getSignaturePayload = do
 putSignaturePayload :: SignaturePayload -> Put
 putSignaturePayload (SigV3 st ctime eok pka ha left16 mpis) = do
     putWord8 3
+    putWord8 5 -- hashlen
     put st
     putWord32be ctime
     putByteString (unEOKI eok)
@@ -1191,7 +1208,9 @@ putSignaturePayload (SigV4 st pka ha hashed unhashed left16 mpis) = do
     putByteString ub
     putWord16be left16
     mapM_ put mpis
-putSignaturePayload _ = fail "Signature version not supported"
+putSignaturePayload (SigVOther pv bs) = do
+    putWord8 pv
+    putByteString bs
 
 -- Stolen from Axman6
 many :: Get a -> Get [a]
