@@ -18,9 +18,10 @@ module Codec.Encryption.OpenPGP.SerializeForSigs (
 import Control.Lens ((^.))
 import Data.ByteString (ByteString)
 import qualified Data.ByteString as B
-import qualified Data.ByteString.Char8 as BC8
 import Data.Serialize (Serialize, put)
 import Data.Serialize.Put (Put, putWord8, putWord16be, putWord32be, putByteString, runPut)
+import qualified Data.Text as T
+import Data.Text.Encoding (encodeUtf8)
 
 import Codec.Encryption.OpenPGP.Internal (PktStreamContext(..), integerToBEBS, pubkeyToMPIs)
 import Codec.Encryption.OpenPGP.Serialize ()
@@ -65,7 +66,7 @@ putUforSigning _ = fail "This should never happen"
 putUIDforSigning :: Pkt -> Put
 putUIDforSigning (UserIdPkt u) = do
     putWord8 0xB4
-    let bs = BC8.pack u
+    let bs = encodeUtf8 . T.pack $ u
     putWord32be . fromIntegral . B.length $ bs
     putByteString bs
 putUIDforSigning _ = fail "This should never happen"
@@ -101,7 +102,7 @@ putKeyForSigning' pkp = do
     putByteString bs
 
 payloadForSig :: SigType -> PktStreamContext -> ByteString
-payloadForSig BinarySig state = (fromPkt (lastLD state))^.literalDataPayload
+payloadForSig BinarySig state = fromPkt (lastLD state)^.literalDataPayload
 payloadForSig CanonicalTextSig state = payloadForSig BinarySig state
 payloadForSig StandaloneSig _ = B.empty
 payloadForSig GenericCert state = kandUPayload (lastPrimaryKey state) (lastUIDorUAt state)
