@@ -1,12 +1,13 @@
--- Verify.hs: OpenPGP (RFC4880) signature verification
--- Copyright © 2012-2013  Clint Adams
+-- Signatures.hs: OpenPGP (RFC4880) signature verification
+-- Copyright © 2012-2014  Clint Adams
 -- This software is released under the terms of the Expat license.
 -- (See the LICENSE file).
 
 module Codec.Encryption.OpenPGP.Signatures (
-   verifySig
- , verify
- , verifyTK
+   verifySigWith
+ , verifyAgainstKeyring
+ , verifyAgainstKeys
+ , verifyTKWith
 ) where
 
 import Control.Monad (guard, liftM2)
@@ -15,7 +16,6 @@ import Crypto.PubKey.HashDescr (HashDescr(..))
 import qualified Crypto.PubKey.DSA as DSA
 import qualified Crypto.PubKey.RSA.PKCS15 as P15
 
-import Control.Lens ((^.))
 import Data.ByteString (ByteString)
 import qualified Data.ByteString as B
 import Data.Either (lefts, rights)
@@ -31,19 +31,19 @@ import Codec.Encryption.OpenPGP.SerializeForSigs (putPartialSigforSigning, putSi
 import Codec.Encryption.OpenPGP.Types
 import Data.Conduit.OpenPGP.Keyring.Instances ()
 
-verifySig :: Keyring -> Pkt -> PktStreamContext -> Maybe UTCTime -> Either String Verification -- FIXME: check expiration here?
-verifySig kr sig@(SignaturePkt (SigV4 st _ _ hs _ _ _)) state mt = do
-    v <- verify kr sig mt (payloadForSig st state)
+verifySigWith :: (Pkt -> Maybe UTCTime -> ByteString -> Either String Verification) -> Pkt -> PktStreamContext -> Maybe UTCTime -> Either String Verification -- FIXME: check expiration here?
+verifySigWith vf sig@(SignaturePkt (SigV4 st _ _ hs _ _ _)) state mt = do
+    v <- vf sig mt (payloadForSig st state)
     _ <- mapM_ (checkIssuer (eightOctetKeyID (_verificationSigner v)) . _sspPayload) hs
     return v
     where
         checkIssuer :: EightOctetKeyId -> SigSubPacketPayload -> Either String Bool
         checkIssuer signer (Issuer i) = if signer == i then Right True else Left "issuer subpacket does not match"
         checkIssuer _ _ = Right True
-verifySig _ _ _ _ = Left "This should never happen."
+verifySigWith _ _ _ _ = Left "This should never happen."
 
-verifyTK :: Keyring -> Maybe UTCTime -> TK -> Either String TK
-verifyTK kr mt key = do
+verifyTKWith :: (Pkt -> PktStreamContext -> Maybe UTCTime -> Either String Verification) -> Maybe UTCTime -> TK -> Either String TK
+verifyTKWith vsf mt key = do
     revokers <- checkRevokers key
     revs <- checkKeyRevocations revokers key
     let uids = filter (\(_, sps) -> sps /= []) . checkUidSigs $ _tkUIDs key -- FIXME: check revocations here?
@@ -53,24 +53,24 @@ verifyTK kr mt key = do
     where
         checkRevokers = Right . concat . rights . map verifyRevoker . filter isRevokerP . _tkRevs
         checkKeyRevocations :: [(PubKeyAlgorithm, TwentyOctetFingerprint)] -> TK -> Either String [SignaturePayload]
-        checkKeyRevocations rs k = Prelude.sequence . concatMap (filterRevs rs) . rights . map (liftM2 fmap (,) (vSig kr)) . _tkRevs $ k
+        checkKeyRevocations rs k = Prelude.sequence . concatMap (filterRevs rs) . rights . map (liftM2 fmap (,) vSig) . _tkRevs $ k
         checkUidSigs :: [(String, [SignaturePayload])] -> [(String, [SignaturePayload])]
-        checkUidSigs = map (\(uid, sps) -> (uid, (rights . map (\sp -> fmap (const sp) (vUid kr (uid, sp)))) sps))
+        checkUidSigs = map (\(uid, sps) -> (uid, (rights . map (\sp -> fmap (const sp) (vUid (uid, sp)))) sps))
         checkUAtSigs :: [([UserAttrSubPacket], [SignaturePayload])] -> [([UserAttrSubPacket], [SignaturePayload])]
-        checkUAtSigs = map (\(uat, sps) -> (uat, (rights . map (\sp -> fmap (const sp) (vUAt kr (uat, sp)))) sps))
+        checkUAtSigs = map (\(uat, sps) -> (uat, (rights . map (\sp -> fmap (const sp) (vUAt (uat, sp)))) sps))
         checkSub :: (Pkt, SignaturePayload, Maybe SignaturePayload) -> [(Pkt, SignaturePayload, Maybe SignaturePayload)]
         checkSub (pkt, sp, mrp) = if revokedSub pkt mrp then [] else checkSub' pkt sp
         revokedSub :: Pkt -> Maybe SignaturePayload -> Bool
         revokedSub _ Nothing = False
-        revokedSub p (Just rp) = vSubSig kr p rp
+        revokedSub p (Just rp) = vSubSig p rp
         checkSub' :: Pkt -> SignaturePayload -> [(Pkt, SignaturePayload, Maybe SignaturePayload)]
-        checkSub' p sp = guard (vSubSig kr p sp) >> return (p, sp, Nothing)
+        checkSub' p sp = guard (vSubSig p sp) >> return (p, sp, Nothing)
         getHasheds (SigV4 _ _ _ ha _ _ _) = ha
         getHasheds _ = []
 	filterRevs :: [(PubKeyAlgorithm, TwentyOctetFingerprint)] -> (SignaturePayload, Verification) -> [Either String SignaturePayload]
 	filterRevs vokers spv = case spv of
                                      (s@(SigV4 SignatureDirectlyOnAKey _ _ _ _ _ _), _) -> [Right s]
-                                     (s@(SigV4 KeyRevocationSig pka _ _ _ _ _), v) -> if any (\(p,f) -> p == pka && f == fingerprint (_verificationSigner v)) vokers then [Left "Key revoked"] else [Right s]
+                                     (s@(SigV4 KeyRevocationSig pka _ _ _ _ _), v) -> if (_verificationSigner v == _tkPKP key) || any (\(p,f) -> p == pka && f == fingerprint (_verificationSigner v)) vokers then [Left "Key revoked"] else [Right s]
 				     _ -> []
         isKeyRevocation (SigV4 KeyRevocationSig _ _ _ _ _ _) = True
         isKeyRevocation _ = False
@@ -80,26 +80,30 @@ verifyTK kr mt key = do
         isRevocationKeySSP _ = False
         isIssuerSSP (SigSubPacket _ (Issuer _)) = True
         isIssuerSSP _ = False
-        vUid :: Keyring -> (String, SignaturePayload) -> Either String Verification
-        vUid keyring (uid, sp) = verifySig keyring (SignaturePkt sp) emptyPSC { lastPrimaryKey = PublicKeyPkt (_tkPKP key), lastUIDorUAt = UserIdPkt uid } mt
-        vUAt :: Keyring -> ([UserAttrSubPacket], SignaturePayload) -> Either String Verification
-        vUAt keyring (uat, sp) = verifySig keyring (SignaturePkt sp) emptyPSC { lastPrimaryKey = PublicKeyPkt (_tkPKP key), lastUIDorUAt = UserAttributePkt uat } mt
-        vSig :: Keyring -> SignaturePayload -> Either String Verification
-        vSig keyring sp = verifySig keyring (SignaturePkt sp) emptyPSC { lastPrimaryKey = PublicKeyPkt (_tkPKP key) } mt
-        vSubSig :: Keyring -> Pkt -> SignaturePayload -> Bool
-        vSubSig keyring sk sp = case verifySig keyring (SignaturePkt sp) emptyPSC { lastPrimaryKey = PublicKeyPkt (_tkPKP key), lastSubkey = sk} mt of
+        vUid :: (String, SignaturePayload) -> Either String Verification
+        vUid (uid, sp) = vsf (SignaturePkt sp) emptyPSC { lastPrimaryKey = PublicKeyPkt (_tkPKP key), lastUIDorUAt = UserIdPkt uid } mt
+        vUAt :: ([UserAttrSubPacket], SignaturePayload) -> Either String Verification
+        vUAt (uat, sp) = vsf (SignaturePkt sp) emptyPSC { lastPrimaryKey = PublicKeyPkt (_tkPKP key), lastUIDorUAt = UserAttributePkt uat } mt
+        vSig :: SignaturePayload -> Either String Verification
+        vSig sp = vsf (SignaturePkt sp) emptyPSC { lastPrimaryKey = PublicKeyPkt (_tkPKP key) } mt
+        vSubSig :: Pkt -> SignaturePayload -> Bool
+        vSubSig sk sp = case vsf (SignaturePkt sp) emptyPSC { lastPrimaryKey = PublicKeyPkt (_tkPKP key), lastSubkey = sk} mt of
                                 Left _ -> False
 				Right _ -> True
         verifyRevoker :: SignaturePayload -> Either String [(PubKeyAlgorithm, TwentyOctetFingerprint)]
         verifyRevoker sp = do
-            _ <- vSig kr sp
+            _ <- vSig sp
             return (map (\(SigSubPacket _ (RevocationKey _ pka fp)) -> (pka, fp)) . filter isRevocationKeySSP $ getHasheds sp)
 
-verify :: Keyring -> Pkt -> Maybe UTCTime -> ByteString -> Either String Verification
-verify kr sig mt payload = do
+verifyAgainstKeyring :: Keyring -> Pkt -> Maybe UTCTime -> ByteString -> Either String Verification
+verifyAgainstKeyring kr sig mt payload = do
     i <- maybe (Left "issuer not found") Right (issuer sig)
     potentialmatches <- if IxSet.null (kr @= i) then Left "pubkey not found" else Right (kr @= i)
-    let allrelevantpkps = filter (\x -> issuer sig == Just (eightOctetKeyID x)) (concatMap (\x -> _tkPKP x:map subPKP (_tkSubs x)) (IxSet.toList potentialmatches))
+    verifyAgainstKeys (IxSet.toList potentialmatches) sig mt payload
+
+verifyAgainstKeys :: [TK] -> Pkt -> Maybe UTCTime -> ByteString -> Either String Verification
+verifyAgainstKeys ks sig mt payload = do
+    let allrelevantpkps = filter (\x -> issuer sig == Just (eightOctetKeyID x)) (concatMap (\x -> _tkPKP x:map subPKP (_tkSubs x)) ks)
     let results = map (\pkp -> verify' sig pkp (hashalgo sig) (finalPayload sig payload)) allrelevantpkps
     case rights results of
         [] -> Left (concatMap (++"/") (lefts results))
@@ -116,9 +120,9 @@ verify kr sig mt payload = do
         verify'' (RSA,mpis) ha pub (RSAPubKey pkey) bs = verify''' (rsaVerify mpis ha pkey bs) pub
         verify'' _ _ _ _ _ = Left "unimplemented key type"
 	verify''' f pub = if f then Right pub else Left "verification failed"
-	dsaVerify mpis ha pkey bs = DSA.verify (dsaTruncate pkey . hashFunction (hashDescr ha)) pkey (dsaMPIsToSig mpis) bs
+	dsaVerify mpis ha pkey = DSA.verify (dsaTruncate pkey . hashFunction (hashDescr ha)) pkey (dsaMPIsToSig mpis)
 	rsaVerify mpis ha pkey bs = P15.verify (hashDescr ha) pkey bs (rsaMPItoSig mpis)
-        dsaMPIsToSig mpis = (DSA.Signature (unMPI (mpis !! 0)) (unMPI (mpis !! 1)))
+        dsaMPIsToSig mpis = DSA.Signature (unMPI (head mpis)) (unMPI (mpis !! 1))
         rsaMPItoSig mpis = integerToBEBS (unMPI (head mpis))
         hashalgo :: Pkt -> HashAlgorithm
         hashalgo (SignaturePkt (SigV4 _ _ ha _ _ _ _)) = ha
