@@ -1,5 +1,5 @@
 -- Types.hs: OpenPGP (RFC4880) data types
--- Copyright © 2012-2013  Clint Adams
+-- Copyright © 2012-2014  Clint Adams
 -- This software is released under the terms of the Expat license.
 -- (See the LICENSE file).
 
@@ -18,6 +18,8 @@ import Data.Char (toLower, toUpper)
 import Data.Data (Data)
 import Data.IxSet (IxSet)
 import Data.List.Split (chunksOf)
+import Data.Monoid ((<>))
+import Data.Ord (comparing)
 import Data.Set (Set)
 import Data.Typeable (Typeable)
 import Data.Word (Word8, Word16, Word32)
@@ -381,8 +383,7 @@ data PKPayload = PKPayload {
     , _v3exp :: V3Expiration
     , _pkalgo :: PubKeyAlgorithm
     , _pubkey :: PKey
-    }
-    deriving (Data, Eq, Show, Typeable)
+    } deriving (Data, Eq, Show, Typeable)
 
 instance Ord PKPayload where
     compare a b = show a `compare` show b -- FIXME: this is ridiculous
@@ -505,14 +506,19 @@ instance FutureVal SigType where
     toFVal 0x50 = ThirdPartyConfirmationSig
     toFVal o = OtherSig o
 
+instance Ord DSA.PublicKey
+instance Ord RSA.PublicKey
+
 data PKey = RSAPubKey RSA.PublicKey
           | DSAPubKey DSA.PublicKey
           | ElGamalPubKey [Integer]
-    deriving (Data, Eq, Show, Typeable)
+	  | UnknownPKey ByteString
+    deriving (Data, Eq, Ord, Show, Typeable)
 
 data SKey = RSAPrivateKey RSA.PrivateKey
           | DSAPrivateKey DSA.PrivateKey
           | ElGamalPrivateKey [Integer]
+	  | UnknownSKey ByteString
     deriving (Data, Eq, Show, Typeable)
 
 newtype Block a = Block {unBlock :: [a]} -- so we can override cereal instance
@@ -552,7 +558,7 @@ data TK = TK {
   } deriving (Data, Eq, Show, Typeable)
 
 instance Ord TK where
-    compare a b = show a `compare` show b -- FIXME: this is ridiculous
+    compare = comparing _tkPKP -- FIXME: is this ridiculous?
 
 type Keyring = IxSet TK
 
@@ -582,7 +588,28 @@ data Pkt = PKESKPkt PacketVersion EightOctetKeyId PubKeyAlgorithm [MPI]
          | SymEncIntegrityProtectedDataPkt PacketVersion ByteString
          | ModificationDetectionCodePkt ByteString
          | OtherPacketPkt Word8 ByteString
-    deriving (Show, Eq, Data, Typeable) -- FIXME
+         | BrokenPacketPkt String Word8 ByteString
+    deriving (Data, Eq, Show, Typeable) -- FIXME
+
+pktTag :: Pkt -> Word8
+pktTag (PKESKPkt {}) = 1
+pktTag (SignaturePkt _) = 2
+pktTag (SKESKPkt {}) = 3
+pktTag (OnePassSignaturePkt {}) = 4
+pktTag (SecretKeyPkt {}) = 5
+pktTag (PublicKeyPkt _) = 6
+pktTag (SecretSubkeyPkt {}) = 7
+pktTag (CompressedDataPkt {}) = 8
+pktTag (SymEncDataPkt _) = 9
+pktTag (MarkerPkt _) = 10
+pktTag (LiteralDataPkt {}) = 11
+pktTag (TrustPkt _) = 12
+pktTag (UserIdPkt _) = 13
+pktTag (PublicSubkeyPkt _) = 14
+pktTag (UserAttributePkt _) = 17
+pktTag (SymEncIntegrityProtectedDataPkt {}) = 18
+pktTag (ModificationDetectionCodePkt _) = 19
+pktTag (OtherPacketPkt t _) = t
 
 data PKESK = PKESK
     { _pkeskPacketVersion :: PacketVersion
@@ -783,6 +810,18 @@ instance Packet OtherPacket where
     toPkt (OtherPacket a b) = OtherPacketPkt a b
     fromPkt (OtherPacketPkt a b) = OtherPacket a b
 
+data BrokenPacket = BrokenPacket
+    { _brokenPacketParseError :: String
+    , _brokenPacketType :: Word8
+    , _brokenPacketPayload :: ByteString
+    } deriving (Data, Eq, Show, Typeable)
+instance Packet BrokenPacket where
+    data PacketType BrokenPacket = BrokenPacketType deriving (Show, Eq)
+    packetType _ = BrokenPacketType
+    packetCode _ = undefined
+    toPkt (BrokenPacket a b c) = BrokenPacketPkt a b c
+    fromPkt (BrokenPacketPkt a b c) = BrokenPacket a b c
+
 data Verification = Verification {
       _verificationSigner :: PKPayload
     , _verificationSignature :: SignaturePayload
