@@ -9,7 +9,10 @@ module Data.Conduit.OpenPGP.Decrypt (
    conduitDecrypt
 ) where
 
+import Control.Monad (when)
 import Control.Monad.IO.Class (MonadIO(..))
+import qualified Control.Monad.Trans.State.Lazy as S
+import qualified Crypto.Hash.SHA1 as SHA1
 import qualified Data.ByteString as B
 import qualified Data.ByteString.Lazy as BL
 import Data.Conduit
@@ -18,7 +21,7 @@ import Data.Conduit.Cereal (conduitGet)
 import Data.Conduit.OpenPGP.Compression (conduitDecompress)
 import qualified Data.Conduit.List as CL
 import Data.Default (Default, def)
-import Data.Maybe (fromJust)
+import Data.Maybe (fromJust, isNothing)
 import Data.Serialize (get)
 
 import Codec.Encryption.OpenPGP.S2K (skesk2Key)
@@ -29,10 +32,11 @@ data RecursorState = RecursorState {
      _depth    :: Int
   , _lastPKESK :: Maybe PKESK
   , _lastSKESK :: Maybe SKESK
+  , _lastLDP   :: Maybe LiteralData
 } deriving (Eq, Show)
 
 instance Default RecursorState where
-    def = RecursorState 0 Nothing Nothing
+    def = RecursorState 0 Nothing Nothing Nothing
 
 type InputCallback m = String -> m BL.ByteString
 
@@ -48,10 +52,16 @@ conduitDecrypt' depth cb = CL.concatMapAccumM push def { _depth = depth }  -- FI
 	    | otherwise = case i of
                        (SKESKPkt {}) -> return (s { _lastSKESK = Just (fromPkt i) }, [])
                        (SymEncDataPkt bs) -> do d <- decryptSEDP (_depth s) cb (fromJust . _lastSKESK $ s) bs
-                                                return (s, d)
+                                                return (processLDPs s d, d)
                        (SymEncIntegrityProtectedDataPkt _ bs) -> do d <- decryptSEIPDP (_depth s) cb (fromJust . _lastSKESK $ s) bs
-                                                                    return (s, d)
+                                                                    return (processLDPs s d, d)
+                       m@(ModificationDetectionCodePkt mdc) -> do when (isNothing (_lastLDP s)) $ fail "MDC with no referent"
+                                                                  when (fmap (SHA1.hash . _literalDataPayload) (_lastLDP s) /= Just mdc) $ fail "MDC indicates tampering"
+                                                                  return (s, [m])
                        p -> return (s, [p])
+        processLDPs s ds = S.execState (mapM_ ldpCheck ds) s
+        ldpCheck l@(LiteralDataPkt {}) = S.get >>= \o -> S.put o { _lastLDP = Just . fromPkt $ l }
+        ldpCheck _ = return ()
 
 decryptSEDP :: (MonadBaseControl IO m, MonadIO m, MonadThrow m, MonadUnsafeIO m) => Int -> InputCallback IO -> SKESK -> B.ByteString -> m [Pkt]
 decryptSEDP depth cb skesk bs = do -- FIXME: this shouldn't pass the whole SKESK
