@@ -1,33 +1,39 @@
 {-# LANGUAGE OverloadedStrings #-}
 
 -- suite.hs: hOpenPGP test suite
--- Copyright © 2012-2013  Clint Adams
+-- Copyright © 2012-2014  Clint Adams
 -- This software is released under the terms of the Expat license.
 -- (See the LICENSE file).
 
-import Test.Framework (defaultMain, testGroup, Test)
-import Test.Framework.Providers.HUnit
+import Test.Tasty (defaultMain, testGroup, TestTree)
+import Test.Tasty.HUnit (testCase, Assertion, assertFailure, assertEqual)
+import Test.Tasty.QuickCheck as QC
 
-import Test.HUnit (Assertion, assertFailure, assertEqual)
-
-import Codec.Encryption.OpenPGP.Serialize ()
+import qualified Data.ByteString as B
+import qualified Data.ByteString.Lazy as BL
+import Codec.Encryption.OpenPGP.Arbitrary ()
 import Codec.Encryption.OpenPGP.Compression (decompressPkt, compressPkts)
+import Codec.Encryption.OpenPGP.Expirations (isTKTimeValid)
 import Codec.Encryption.OpenPGP.Fingerprint (eightOctetKeyID, fingerprint)
+import Codec.Encryption.OpenPGP.KeySelection (parseFingerprint)
+import Codec.Encryption.OpenPGP.SecretKey (decryptPrivateKey, encryptPrivateKey)
+import Codec.Encryption.OpenPGP.Serialize ()
+import Codec.Encryption.OpenPGP.Signatures (verifyTKWith, verifySigWith, verifyAgainstKeys)
 import Codec.Encryption.OpenPGP.Types
+import Control.Error.Util (isRight)
+import Crypto.PubKey.RSA (PrivateKey(private_pub))
 import Data.Conduit.Cereal (conduitGet)
 import Data.Conduit.OpenPGP.Compression (conduitCompress, conduitDecompress)
 import Data.Conduit.OpenPGP.Decrypt (conduitDecrypt)
 import Data.Conduit.OpenPGP.Keyring (conduitToTKs, conduitToTKsDropping, sinkKeyringMap)
 import Data.Conduit.OpenPGP.Verify (conduitVerify)
-import Codec.Encryption.OpenPGP.Serialize ()
-
-import qualified Data.ByteString as B
-import qualified Data.ByteString.Lazy as BL
 import Data.IxSet ((@=), getOne)
 import Data.Maybe (isJust)
 import Data.Serialize (get, put)
 import Data.Serialize.Get (runGet, Get)
 import Data.Serialize.Put (runPut)
+import Data.Text (Text)
+import Data.Time.Clock.POSIX (posixSecondsToUTCTime)
 
 import qualified Data.Conduit as DC
 import qualified Data.Conduit.Binary as CB
@@ -50,12 +56,12 @@ testSerialization fpr = do
 testCompression :: FilePath -> Assertion
 testCompression fpr = do
     bs <- B.readFile $ "tests/data/" ++ fpr
-    let firstpass = fmap (concatMap decompressPkt) . fmap unBlock . runGet get $ bs
+    let firstpass = fmap (concatMap decompressPkt . unBlock) . runGet get $ bs
     case firstpass of
         Left _ -> assertFailure $ "First pass failed on " ++ fpr
         Right [] -> assertFailure $ "First pass of " ++ fpr ++ " decoded to nothing."
         Right packs -> do let roundtrip = runPut $ put . Block $ [compressPkts ZIP packs]
-                          let secondpass = fmap (concatMap decompressPkt) . fmap unBlock . runGet get $ roundtrip
+                          let secondpass = fmap (concatMap decompressPkt . unBlock) . runGet get $ roundtrip
                           if secondpass == Right [] then
                               assertFailure $ "Second pass of " ++ fpr ++ " decoded to nothing."
                           else
@@ -90,6 +96,19 @@ testVerifyMessage keyring message issuers = do
     let verification' = map (fmap (fingerprint . _verificationSigner)) verification
     assertEqual (keyring ++ " for " ++ message) (map Right issuers) verification'
 
+testKeysSelfVerification :: Bool -> FilePath -> Assertion
+testKeysSelfVerification expectsuccess keyfile = do
+    ks <- DC.runResourceT $ CB.sourceFile ("tests/data/" ++ keyfile) DC.$= conduitGet get DC.$= conduitToTKs DC.$$ CL.consume
+    let verifieds = mapM (verifyTKWith (verifySigWith (verifyAgainstKeys ks)) Nothing) ks
+    assertEqual (keyfile ++ " self-verification") expectsuccess (isRight verifieds)
+
+testKeysExpiration :: Bool -> FilePath -> Assertion
+testKeysExpiration expectsuccess keyfile = do
+    ks <- DC.runResourceT $ CB.sourceFile ("tests/data/" ++ keyfile) DC.$= conduitGet get DC.$= conduitToTKs DC.$$ CL.consume
+    let Right verifieds = mapM (verifyTKWith (verifySigWith (verifyAgainstKeys ks)) Nothing) ks
+        tvalid = all (isTKTimeValid (posixSecondsToUTCTime (realToFrac 1400000000))) verifieds
+    assertEqual (keyfile ++ " key expiration") expectsuccess tvalid
+
 -- This needs a lot of work
 testSymmetricEncryption :: FilePath -> FilePath -> B.ByteString -> Assertion
 testSymmetricEncryption encfile passfile cleartext = do
@@ -112,8 +131,32 @@ testSymmetricEncryption encfile passfile cleartext = do
           fakeCallback :: BL.ByteString -> String -> IO BL.ByteString
           fakeCallback = const . return
 
-tests :: [Test]
-tests = [
+testSecretKeyDecryption :: FilePath -> FilePath -> Assertion
+testSecretKeyDecryption keyfile passfile = do
+  passphrase <- BL.readFile $ "tests/data/" ++ passfile
+  kr <- DC.runResourceT $ CB.sourceFile ("tests/data/" ++ keyfile) DC.$= conduitGet get DC.$$ CL.consume
+  let SecretKey pkp ska = fromPkt . head $ kr
+      SUUnencrypted (RSAPrivateKey dpk) _ = decryptPrivateKey (pkp, ska) passphrase  -- FIXME: better API for multiple keytypes
+      RSAPubKey pk = _pubkey pkp
+  assertEqual "private key matches public key" pk (private_pub dpk)
+
+-- FIXME: this should be reworked either with tasty-golden or some other form of sanity
+testSecretKeyEncryption :: FilePath -> FilePath -> Assertion
+testSecretKeyEncryption keyfile passfile = do
+  passphrase <- BL.readFile $ "tests/data/" ++ passfile
+  kr <- DC.runResourceT $ CB.sourceFile ("tests/data/" ++ keyfile) DC.$= conduitGet get DC.$$ CL.consume
+  gkr <- DC.runResourceT $ CB.sourceFile ("tests/data/" ++ "aes256-sha512.seckey") DC.$= conduitGet get DC.$$ CL.consume
+  let SecretKey pkp ska = fromPkt . head $ kr
+      newska = encryptPrivateKey "\226~\197\a\202#\"G" "\187\219\253I\236\204\t5D\196\NAK>;\202\185\t" ska passphrase
+      newtruck = toPkt (SecretKey pkp newska):tail kr
+  assertEqual "encrypted private key matches golden file" gkr newtruck
+
+
+tests :: TestTree
+tests = testGroup "Tests" [properties, unitTests]
+
+unitTests :: TestTree
+unitTests = testGroup "Unit Tests" [
   testGroup "Serialization group" [
      testCase "000001-006.public_key" (testSerialization "000001-006.public_key")
    , testCase "000002-013.user_id" (testSerialization "000002-013.user_id")
@@ -202,6 +245,7 @@ tests = [
    , testCase "uncompressed-ops-dsa.gpg" (testSerialization "uncompressed-ops-dsa.gpg")
    , testCase "uncompressed-ops-rsa.gpg" (testSerialization "uncompressed-ops-rsa.gpg")
    , testCase "simple.seckey" (testSerialization "simple.seckey")
+   , testCase "v3-genericcert.sig" (testSerialization "v3-genericcert.sig")
    ],
   testGroup "KeyID/fingerprint group" [
      testCase "v3 key" (testKeyIDandFingerprint "v3.key" "C7261095/CBD9 F412 6807 E405 CC2D  2712 1DF5 E86E  ")
@@ -217,23 +261,32 @@ tests = [
    -- FIXME: should count keys in rings
    ],
   testGroup "Message verification group" [
-     testCase "uncompressed-ops-dsa" (testVerifyMessage "pubring.gpg" "uncompressed-ops-dsa.gpg" ([fp "1EB2 0B2F 5A5C C3BE AFD6  E5CB 7732 CF98 8A63 EA86"]))
-   , testCase "uncompressed-ops-dsa-sha384" (testVerifyMessage "pubring.gpg" "uncompressed-ops-dsa-sha384.txt.gpg" ([fp "1EB2 0B2F 5A5C C3BE AFD6  E5CB 7732 CF98 8A63 EA86"]))
-   , testCase "uncompressed-ops-rsa" (testVerifyMessage "pubring.gpg" "uncompressed-ops-rsa.gpg" ([fp "CB79 3345 9F59 C70D F1C3  FBEE DEDC 3ECF 689A F56D"]))
-   , testCase "compressedsig" (testVerifyMessage "pubring.gpg" "compressedsig.gpg" ([fp "421F 28FE AAD2 22F8 56C8  FFD5 D4D5 4EA1 6F87 040E"]))
-   , testCase "compressedsig-zlib" (testVerifyMessage "pubring.gpg" "compressedsig-zlib.gpg" ([fp "421F 28FE AAD2 22F8 56C8  FFD5 D4D5 4EA1 6F87 040E"]))
-   , testCase "compressedsig-bzip2" (testVerifyMessage "pubring.gpg" "compressedsig-bzip2.gpg" ([fp "421F 28FE AAD2 22F8 56C8  FFD5 D4D5 4EA1 6F87 040E"]))
+     testCase "uncompressed-ops-dsa" (testVerifyMessage "pubring.gpg" "uncompressed-ops-dsa.gpg" [fp "1EB2 0B2F 5A5C C3BE AFD6  E5CB 7732 CF98 8A63 EA86"])
+   , testCase "uncompressed-ops-dsa-sha384" (testVerifyMessage "pubring.gpg" "uncompressed-ops-dsa-sha384.txt.gpg" [fp "1EB2 0B2F 5A5C C3BE AFD6  E5CB 7732 CF98 8A63 EA86"])
+   , testCase "uncompressed-ops-rsa" (testVerifyMessage "pubring.gpg" "uncompressed-ops-rsa.gpg" [fp "CB79 3345 9F59 C70D F1C3  FBEE DEDC 3ECF 689A F56D"])
+   , testCase "compressedsig" (testVerifyMessage "pubring.gpg" "compressedsig.gpg" [fp "421F 28FE AAD2 22F8 56C8  FFD5 D4D5 4EA1 6F87 040E"])
+   , testCase "compressedsig-zlib" (testVerifyMessage "pubring.gpg" "compressedsig-zlib.gpg" [fp "421F 28FE AAD2 22F8 56C8  FFD5 D4D5 4EA1 6F87 040E"])
+   , testCase "compressedsig-bzip2" (testVerifyMessage "pubring.gpg" "compressedsig-bzip2.gpg" [fp "421F 28FE AAD2 22F8 56C8  FFD5 D4D5 4EA1 6F87 040E"])
    ],
   testGroup "Certificate verification group" [
-     testCase "userid" (testVerifyMessage "pubring.gpg" "minimized.gpg" ([fp "421F 28FE AAD2 22F8 56C8  FFD5 D4D5 4EA1 6F87 040E"]))
-   , testCase "subkey" (testVerifyMessage "pubring.gpg" "subkey.gpg" ([fp "421F 28FE AAD2 22F8 56C8  FFD5 D4D5 4EA1 6F87 040E"]))
-   , testCase "primary key binding" (testVerifyMessage "signing-subkey.gpg" "primary-binding.gpg" ([fp "ED1B D216 F70E 5D5F 4444  48F9 B830 F2C4 83A9 9AE5"]))
-   , testCase "attribute" (testVerifyMessage "pubring.gpg" "uat.gpg" ([fp "421F 28FE AAD2 22F8 56C8  FFD5 D4D5 4EA1 6F87 040E"]))
-   , testCase "primary key revocation" (testVerifyMessage "pubring.gpg" "prikey-rev.gpg" ([fp "421F 28FE AAD2 22F8 56C8  FFD5 D4D5 4EA1 6F87 040E"]))
-   , testCase "subkey revocation" (testVerifyMessage "pubring.gpg" "subkey-rev.gpg" ([fp "421F 28FE AAD2 22F8 56C8  FFD5 D4D5 4EA1 6F87 040E"]))
-   , testCase "6F87040E" (testVerifyMessage "pubring.gpg" "6F87040E.pubkey" ([fp "421F 28FE AAD2 22F8 56C8  FFD5 D4D5 4EA1 6F87 040E", fp "CB79 3345 9F59 C70D F1C3  FBEE DEDC 3ECF 689A F56D", fp "AF95 E4D7 BAC5 21EE 9740  BED7 5E9F 1523 4132 62DC"]))
-   , testCase "6F87040E-cr" (testVerifyMessage "pubring.gpg" "6F87040E-cr.pubkey" ([fp "AF95 E4D7 BAC5 21EE 9740  BED7 5E9F 1523 4132 62DC", fp "AF95 E4D7 BAC5 21EE 9740  BED7 5E9F 1523 4132 62DC", fp "421F 28FE AAD2 22F8 56C8  FFD5 D4D5 4EA1 6F87 040E", fp "CB79 3345 9F59 C70D F1C3  FBEE DEDC 3ECF 689A F56D", fp "AF95 E4D7 BAC5 21EE 9740  BED7 5E9F 1523 4132 62DC"]))
-   , testCase "simple RSA secret key" (testVerifyMessage "pubring.gpg" "simple.seckey" ([fp "421F 28FE AAD2 22F8 56C8  FFD5 D4D5 4EA1 6F87 040E"]))
+     testCase "userid" (testVerifyMessage "pubring.gpg" "minimized.gpg" [fp "421F 28FE AAD2 22F8 56C8  FFD5 D4D5 4EA1 6F87 040E"])
+   , testCase "subkey" (testVerifyMessage "pubring.gpg" "subkey.gpg" [fp "421F 28FE AAD2 22F8 56C8  FFD5 D4D5 4EA1 6F87 040E"])
+   , testCase "primary key binding" (testVerifyMessage "signing-subkey.gpg" "primary-binding.gpg" [fp "ED1B D216 F70E 5D5F 4444  48F9 B830 F2C4 83A9 9AE5"])
+   , testCase "attribute" (testVerifyMessage "pubring.gpg" "uat.gpg" [fp "421F 28FE AAD2 22F8 56C8  FFD5 D4D5 4EA1 6F87 040E"])
+   , testCase "primary key revocation" (testVerifyMessage "pubring.gpg" "prikey-rev.gpg" [fp "421F 28FE AAD2 22F8 56C8  FFD5 D4D5 4EA1 6F87 040E"])
+   , testCase "subkey revocation" (testVerifyMessage "pubring.gpg" "subkey-rev.gpg" [fp "421F 28FE AAD2 22F8 56C8  FFD5 D4D5 4EA1 6F87 040E"])
+   , testCase "6F87040E" (testVerifyMessage "pubring.gpg" "6F87040E.pubkey" [fp "421F 28FE AAD2 22F8 56C8  FFD5 D4D5 4EA1 6F87 040E", fp "CB79 3345 9F59 C70D F1C3  FBEE DEDC 3ECF 689A F56D", fp "AF95 E4D7 BAC5 21EE 9740  BED7 5E9F 1523 4132 62DC"])
+   , testCase "6F87040E-cr" (testVerifyMessage "pubring.gpg" "6F87040E-cr.pubkey" [fp "AF95 E4D7 BAC5 21EE 9740  BED7 5E9F 1523 4132 62DC", fp "AF95 E4D7 BAC5 21EE 9740  BED7 5E9F 1523 4132 62DC", fp "421F 28FE AAD2 22F8 56C8  FFD5 D4D5 4EA1 6F87 040E", fp "CB79 3345 9F59 C70D F1C3  FBEE DEDC 3ECF 689A F56D", fp "AF95 E4D7 BAC5 21EE 9740  BED7 5E9F 1523 4132 62DC"])
+   , testCase "simple RSA secret key" (testVerifyMessage "pubring.gpg" "simple.seckey" [fp "421F 28FE AAD2 22F8 56C8  FFD5 D4D5 4EA1 6F87 040E"])
+   ],
+  testGroup "Key verification group" [
+      testCase "6F87040E pubkey" (testKeysSelfVerification True "6F87040E.pubkey")
+   ,  testCase "revoked pubkey" (testKeysSelfVerification False "revoked.pubkey")
+   ,  testCase "expired pubkey" (testKeysSelfVerification True "expired.pubkey")
+   ],
+  testGroup "Key expiration group" [
+      testCase "6F87040E pubkey" (testKeysExpiration True "6F87040E.pubkey")
+   ,  testCase "expired pubkey" (testKeysExpiration False "expired.pubkey")
    ],
   testGroup "Compression group" [
      testCase "compressedsig.gpg" (testCompression "compressedsig.gpg")
@@ -268,14 +321,35 @@ tests = [
    , testCase "Symmetric Encryption iterated-salted S2K SHA1 AES256" (testSymmetricEncryption "encryption-sym-aes256.gpg" "symmetric-password.txt" "test\n")
    , testCase "Symmetric Encryption simple S2K SHA1 Twofish" (testSymmetricEncryption "encryption-sym-twofish-s2k0.gpg" "symmetric-password.txt" "test\n")
    , testCase "Symmetric Encryption iterated-salted S2K SHA1 Twofish" (testSymmetricEncryption "encryption-sym-twofish.gpg" "symmetric-password.txt" "test\n")
+   ],
+  testGroup "Encrypted secret keys" [
+     testCase "SUSSHA1 CAST5 IteratedSalted SHA1 RSA" (testSecretKeyDecryption "simple.seckey" "pki-password.txt")
+   , testCase "SUS16bit CAST5 IteratedSalted SHA1 RSA" (testSecretKeyDecryption "16bitcksum.seckey" "pki-password.txt")
+   , testCase "SUSSHA1 AES256 IteratedSalted SHA512 RSA" (testSecretKeyDecryption "aes256-sha512.seckey" "pki-password.txt")
+   ],
+  testGroup "Encrypting secret keys" [
+     testCase "SUSSHA1 AES256 IteratedSalted SHA512 RSA" (testSecretKeyEncryption "unencrypted.seckey" "pki-password.txt")
    ]
  ]
 
+properties :: TestTree
+properties = testGroup "Properties" [qcProps]
+
+qcProps :: TestTree
+qcProps = testGroup "(checked by QuickCheck)"
+  [ QC.testProperty "PKESK packet serialization-deserialization" $
+    \pkesk -> Right (pkesk :: PKESK) == runGet get (runPut (put pkesk))
+  , QC.testProperty "Signature packet serialization-deserialization" $
+    \sig -> Right (sig :: Signature) == runGet get (runPut (put sig))
+  , QC.testProperty "UserId packet serialization-deserialization" $
+    \uid -> Right (uid :: UserId) == runGet get (runPut (put uid))
+  ]
+
 cgp :: DC.Conduit B.ByteString (DC.ResourceT IO) Pkt
 cgp = conduitGet (get :: Get Pkt)
 
-fp :: String -> TwentyOctetFingerprint
-fp = read
+fp :: Text -> TwentyOctetFingerprint
+fp = either error id . parseFingerprint
 
 main :: IO ()
 main = defaultMain tests
